/*		 ITI-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: ampcvReg.h,v 7.0 2000/10/25 09:07:57 cvs Exp $
 *
 */

#ifndef _AMPCVREG_H
#define _AMPCVREG_H

#if defined(linux) && defined(__KERNEL__) || \
    defined(__NetBSD__) && defined(_KERNEL) || \
    !defined(OS_DEFINED) || defined(__CONFIG__)

/* PCVision Hardware Reference Manual; Imaging Technology Inc. */
/* Document Number 47-H60003-01; December 1997, revised June 1998 */

/* R-O   Read-Only Register */
/* W-O   Write-Only Register */
/* R/W   Read/Write Register */
/* R/WC Clear on Write to Register */
/* R/W1C Clear on Write One to Register */
/* R/W0C Clear on Write Zero to Register */

/* PCVision Register Set (16bit access only), pages 3-48 to 3-87 */
/* ************************************************************* */

#ifdef __cplusplus
extern "C" {
#endif

#define PV_REG_SIZE		32	/* number of registers */

#define PV_RES_INIT		0x0000	/* reset reserved bits to zero */

#define PV_REV_MIN		0x10	/* pci revision id starts from this
					   value and defines also the module
					   revision */
#define PV_REV_A		0x13	/* first board layout */
#define PV_REV_B		0x14	/* second board layout */
					/* variable scan support */
					/* back-to-back trigger */

#define PV_DISABLE		0
#define PV_ENABLE		1

/* Module ID */
typedef union {
  u_short all;
  struct {
    u_short rev		:3;	/* Revision R-O */
    u_short type	:5;	/* Board Type R-O */
    u_short		:8;
  } b;
} pv_modid_t;

/* Select Function of ADCDATA Register R/W */
#define PV_LUTADDR_MASK	0x00FF
#define PV_LUTADDR_BASE	0
#define PV_LUTADDR_CONT 0
#define PV_LUTADDR_NREF	1	/* Write Negative Reference Voltage */
#define PV_LUTADDR_PREF	2	/* Write Positive Reference Voltage */

/* Input Lookup Table Data R/W */
#define PV_MIN_LUTDATA	0
#define PV_MAX_LUTDATA	255

/* ADC DATA R/W */

/* Negative Reference R/W */
#define PV_MIN_NREF	0
#define PV_MAX_NREF	255
#define PV_NREF_INIT	10	/* CV-M1 with scale 1.5 */ 
#define PV_NREF_MASK	0x00FC	/* REV-B boards have a resolution
				   of 64 steps / REV-A boards 256 steps */
				/* so we cut off least 2 bits */

/* Positive Reference R/W */
#define PV_MIN_PREF	0
#define PV_MAX_PREF	255
#define PV_PREF_INIT	169	/* CV-M1 with scale 1.5 */ 
#define PV_PREF_MASK	0x00FC	/* REV-B boards have a resolution
				   of 64 steps / REV-A boards 256 steps */
				/* so we cut off least 2 bits */

/* Horizontal Line Rate R/W */
/* [(Total Line Time / 2) / Pixel Clock Rate] - 1 */
#define PV_HTOTAL_MASK	0x03FF
#define PV_MIN_HTOTAL	1
#define PV_MAX_HTOTAL	2048
#define PV_HTOTAL_INIT	910 /* 907 */	/* CCIR: 64us * 14.18MHz clocks */

/* Horizontal Synchronization Pulse Width R/W */
/* (Synchronization Pulse Time / Pixel Clock Rate) - 1 */
#define PV_HSYNCPW_MASK	0x00FF
#define PV_MIN_HSYNCPW	1
#define PV_MAX_HSYNCPW	256
#define PV_HSYNCPW_INIT	68 /* 67 */	/* CCIR: 4.7us * 14.18MHz clocks */	

/* Horizontal Equialization Pulse Width R/W */
/* (Equialization Pulse Time / Pixel Clock Rate) - 1 */
#define PV_HEQUIPW_MASK	0x00FF
#define PV_MIN_HEQUIPW	1
#define PV_MAX_HEQUIPW	256
#define PV_HEQUIPW_INIT	33 /* 33 */	/* CCIR: 2.35us * 14.18MHz clocks */

/* Horizontal Serration Pulse Width R/W */
/* {[(Total Line Time / 2) - Serration Pulse Time] / Pixel Clock Rate) - 1 */
#define PV_HSERRPW_MASK	0x003FF
#define PV_MIN_HSERRPW	1
#define PV_MAX_HSERRPW	1024
#define PV_HSERRPW_INIT	389 /* 387 */	/* CCIR: (64us/2 - 4.7us) * 14.18MHz clocks */ 

/* Vertical Frame Rate R/W */
/* (Total Lines per Frame - 0.5) * 2 */
#define PV_VTOTAL_MASK	0x07FF
#define PV_MIN_VTOTAL	1
#define PV_MAX_VTOTAL	2048
#define PV_VTOTAL_INIT	525 /* 625 */	/* CCIR: 625 lines */

/* Vertical Synchronization Pulse Width R/W */
/* (Number of Lines in Synchronizatzion - 0.5) * 2 */
typedef union {
  u_short all;
  struct {
    u_short vsyncpw	:5;
#define PV_VSYNCPW_MASK	0x001F
#define PV_MIN_VSYNCPW	1
#define PV_MAX_VSYNCPW	32
#define PV_VSYNCPW_INIT	5	/* CCIR:  2.5 lines */
    u_short		:1;
    u_short hsyncpol	:1;	/* HSYNC Output Polarity Select R/W */
#define PV_HS_ACT_LOW	0	/* HSYNC is Active Low */
#define PV_HS_ACT_HIGH	1	/* HSYNC is Active High */
    u_short vsyncpol	:1;	/* VSYNC Output Polarity Select R/W */
#define PV_VS_ACT_LOW	0	/* VSYNC is Active Low */
#define PV_VS_ACT_HIGH	1	/* VSYNC is Active High */
    u_short		:8;
  } b;
} pv_vsyncpw_t;

/* Vertical Gate Start R/W */
/* (Total Lines per Frame - Lines in Front Porch - 0.5) * 2 */
#define PV_VGSTART_MASK	0x07FF
#define PV_MIN_VGSTART	1
#define PV_MAX_VGSTART	2048
#define PV_VGSTART_INIT	519 /* 310 */	/* CCIR: 312.5 - 2.5 lines */

/* Vertical Gate Width R/W */
/* (Lines from VSYNC BEGIN to VGATE END - 0.5) * 2 */
#define PV_VGWIDTH_MASK	0x007F
#define PV_MIN_VGWIDTH	1
#define PV_MAX_VGWIDTH	128
#define PV_VGWIDTH_INIT	12 /* 5 */	/* CCIR: 5 */

/* Horizontal Offset R/W */
/* Number of Pixels from HSYNC Edge to Begin of Active Viedo - 1 */
#define PV_HOFFSET_MASK	0x03FF
#define PV_MIN_HOFFSET	1
#define PV_MAX_HOFFSET	1024
#define PV_HOFFSET_INIT	160 /* 148 */	/* CCIR: (4.7us+5.8us) / 64 us * 907 clocks */ 

/* Horizontal Active R/W */
/* Number of Pixels from HSYNC Edge to End of Active Video - 1 */
#define PV_HACTIVE_MASK	0x07FF
#define PV_MIN_HACTIVE	1
#define PV_MAX_HACTIVE	2048
#define PV_HACTIVE_INIT	926 /* 884 */	/* CCIR: 148 + (51.95us / 64us * 907) clocks */

/* Vertical Offset R/W */
/* Number of Pixels from VSYNC Edge to Begin of Active Video - 1 */
#define PV_VOFFSET_MASK	0x03FF
#define PV_MIN_VOFFSET	1
#define PV_MAX_VOFFSET	1024
#define PV_VOFFSET_INIT	22 /* 22 */	/* CCIR: 2.5 + 19.5 lines */

/* Vertical Active R/W */
/* Number of Pixels from VSYNC Edge to End of Active Video - 1 */
#define PV_VACTIVE_MASK	0x03FF
#define PV_MIN_VACTIVE	1
#define PV_MAX_VACTIVEA	1024	/* REV-A boards can address up to 1024 lines */
#define PV_MAX_VACTIVEB	4096	/* REV-B boards can address up to 4096 lines */
#define PV_VACTIVE_INIT	309 /* 308 */	/* CCIR: 22 + 572 / 2 lines */

/* Strobe Delay R/W */
/* Number of Lines to Delay Strobe - 1 */
typedef union {
  u_short all;
  struct {
    u_short strbdly	:6;
#define PV_STRBDLY_MASK	0x003F
#define PV_MIN_STRBDLY	1
#define PV_MAX_STRBDLY	64
#define PV_STRBDLY_INIT	0x0001	/* Strobe Delay Offset = 1 */	
    u_short lenpol	:1;	/* REV-B Only!*/
    				/* Line Enable Input Polarity Select R/W */
#define PV_LP_ACT_HIGH	0	/* Rising Edge Enables Horizontal Timing */
#define PV_LP_ACT_LOW	1	/* Falling Edge Enables Horizontal Timing */
    u_short fenpol	:1;	/* REV-B Only! */
				/* Frame Enable Input Polarity Select R/W */
#define PV_FP_ACT_HIGH	0	/* Rising Edge Enables Vertical Timing */
#define PV_FP_ACT_LOW	1	/* Falling Edge Enables Vertical Timing */
    u_short		:8;
  } b;
} pv_strbdly_t;

/* Frame Reset Delay R/W */
/* Number o Lines to Delay from Trigger Input - 1 */
typedef union {
  u_short all;
  struct {
    u_short froff	:6;
#define PV_FROFF_MASK	0x03F
#define PV_MIN_FROFF	1
#define PV_MAX_FROFFA	64	/* REV-A boards max time is 64 clock cycles */
#define PV_MAX_FROFFB	256	/* REV-B boards max time is 256 clock cycles */
#define PV_FROFF_INIT	1	/* Frame Reset Offset = 1 */
    u_short		:3;
    u_short frstonv	:1;
    u_short frstmd	:1;
    u_short frstpol	:1;
#define PV_FR_ACT_HIGH	0	/* Frame Reset Pulse Active High */
#define PV_FR_ACT_LOW	1	/* Frame Reset Pulse Active Low */
    u_short frstsz	:1;
#define PV_FRST_FROFF	0	/* Frame Rst Pulse Active for FROFF Period */
#define PV_FRST_LINE	1	/* Frame Rst Pulse Active for one Line */
    u_short		:3;
  } b;
} pv_frstdly_t;

/* Control 1 */
typedef union {
  u_short all;
#define PV_CON1_INIT	0x0156
  struct {
    u_short syncsel	:1;	/* Sync Output Select R/W */
#define PV_HSYNC_OUT	0	/* PTG generates horizontal SYNC output */
#define PV_CSYNC_OUT	1	/* PTG generates composite SYNC output */
    u_short chsyncen	:1;	/* Horiz./Comp. Sync Output Enable R/W */ 
#define PV_CHSYNC_EN	0	/* this is different, we can't use */
#define PV_CHSYNC_DIS	1	/* the normal DISABLE(0) ENABLE(1) */
    u_short vsyncen	:1;	/* Vertical Sync Output Enable R/W */
#define PV_VSYNC_EN	0	/* this is different, we can't use */
#define PV_VSYNC_DIS	1	/* the normal DISABLE(0) ENABLE(1) */
    u_short clksrc	:1;	/* Clock Source Select R/W */    
#define PV_CLKSRC_PLL	0	/* Phase Locked Loop Mode */
#define PV_FIELD_STRIP  0	/* REV-B VS-Mode: Strip Field from Video */
#define PV_CLKSRC_XTAL	1	/* Crystal Mode */
#define PV_FIELD_L_FEN	1	/* REV-B VS-Mode: Detect Field from LEN/FEN */
    u_short lpfsel	:1;	/* Low Pass Filter Select R/W */ 
#define PV_LPF_BYPASS	0	/* Filter disabled (bypassed */
#define PV_LPF_6_0	1	/* Filter 6MHz cut off -3dB */
    u_short vscan	:1;	/* REV-B Only! */
    				/* VariableScan Mode Enable R/W */
    u_short iscale	:1;	/* Video Input Scale Factor R/W */
#define PV_SCALE_1_0	0	/* Scale Video by 1.0 */
#define PV_SCALE_1_5	1	/* Scale Video by 1.5 */
    u_short vclkpol	:1;	/* REV-B Only! */
    				/* VariableScan Clock Polarity */
#define PV_CP_ACT_HIGH	0	/* Disable Clock Input Invert */
#define PV_CP_ACT_LOW	1	/* Enable Clock Input Invert */
    u_short smode	:1;	/* Scan Mode R/W */
#define PV_NON_ILACE	0	/* Non - Interlaced Camera */
#define PV_ILACE	1	/* Interlaced Camera */
    u_short camsel	:2;	/* Camera Select R/W */
#define PV_CAM_0	0	/* Camera 0 enabled */
#define PV_CAM_1	1	/* Camera 1 enabled */
#define PV_CAM_2	2	/* Camera 2 enabled */
#define PV_CAM_3	3	/* Camera 3 enabled */
    u_short fldshft	:1;	/* Field Shift Mode R/W */
    u_short		:4;
  } b;
} pv_con1_t;

/* Control 2 */
typedef union {
  u_short all;
  struct {
    u_short pllcs	:1;	/* PLL Chip Select R/W */
#define PV_PLL_PROG	0	/* PLL in program mode */
#define PV_PLL_NORM	1	/* PLL in normal operation mode */
    u_short einpol	:1;	/* External Input Polarity R/W */
#define PV_EI_ACT_HIGH	0	/* External Inputs Active High (irq at L->H) */
#define PV_EI_ACT_LOW	1	/* External Inputs Active Low (irq at H->L) */
    u_short ein0ien	:1;	/* External Input 0 Interrupt Enable R/W */
    u_short ein1ien	:1;	/*                1                      */ 
    u_short ein0	:1;	/* External Input 0 Status R-O */
    u_short ein1	:1;	/*                1            */
    u_short eout0	:1;	/* External Output 0 Control R/W */
    u_short eout1	:1;	/*                 1             */
    u_short sotint	:1;	/* Start of Trigger Interrupt Status R/WC */
    u_short eotint	:1;	/* End of Trigger Interrupt Status R/WC */
    u_short ein0int	:1;	/* External Input 0 Interrupt Status */
    u_short ein1int	:1;	/*                1                  */
#define PV_IRQ_TEST_MASK	0x0F00	/* here are intr. status bits
					   see xcr_test/clear_irq */
#define PV_IRQ_TEST_SHIFT	8	/* this moves the bits to the
					   right places see itifgInt.h */
    u_short		:4;
  } b;
} pv_con2_t;

/* Trigger Control */

typedef union {
  u_short all;
  struct {
    u_short trigen	:1;	/* Trigger Enable R/W */
    u_short		:2;
    u_short trigcy	:1;	/* Trigger Cycle Status R-O */
#define PV_IN_TRIGCY	0	/* Trigger Cycle in Progress */
#define PV_OUT_TRIGCY	1	/* No Trigger Cycle in Progress */
    u_short sotien	:1;	/* Start of Trigger Interrupt Eanble R/W */
    u_short eotien	:1;	/* End of Trigger Interrupt Enable R/W */
    u_short 		:3;
    u_short trigsrc	:1;	/* Trigger Source Select R/W */
#define PV_TRIG_EXT	0	/* External Trigger */
#define PV_TRIG_SOFT	1	/* Software Trigger */
    u_short trigpol	:1;	/* Trigger Polarity Select R/W */
#define PV_TP_ACT_LOW	0	/* External Trigger Pulse Active Low R/W */
#define PV_TP_ACT_HIGH	1	/* External Trigger Pulse Active High R/W */
    u_short		:1;
    u_short strbmd	:1;	/* Strobe Mode Select R/W */
#define PV_STRBMD_SLOW	0	/* (Trigger on Frame) slow strobe */
#define PV_STRBMD_FAST	1	/* (Trigger on Frame) fast strobe */
    u_short strben	:1;	/* REV-B Only */
    				/* Strobe Enable R/W */
#define PV_STRBMD_EN	0	/* this is different, we can't use */
#define PV_STRBMD_DIS	1	/* the normal DISABLE(0) ENABLE(1) */
    u_short strbpol	:1;	/* Strobe Polarity Select R/W */
#define PV_SP_ACT_HIGH	0	/* Strobe Pulse Active High */
#define PV_SP_ACT_LOW	1	/* Strobe Pulse Active Low */
    u_short skpfldmd	:1;	/* Skip Field Mode */
  } b;
} pv_trigcon_t;

/* Software Trigger W-O */

/* Programmable Clamp */
typedef union {
  u_short all;
#define PV_P_CLAMP_INIT	0x0010
				
  struct {
    u_short ccnt		:4;	/* Clamp Delay Count R/W */
#define PV_MIN_CCNT	0
#define PV_MAX_CCNT	30	/* resolution is 2 pixel clocks ! */
#define PV_CCNT_INIT	0	/* Clamp Delay Count = 0 */
    u_short pclmpol	:1;	/* Clamp Pulse Polarity Select */
#define PV_CLMP_INVERT	0	/* Invert HSYNC Pulse */
#define PV_CLMP_NO_INV	1	/* NoInvert HSYNC Pulse */
    u_short clmpsrc	:1;
#define PV_CLMP_STRIP	0	/* Use Sync Stripper Clamp Pulse */
#define PV_CLMP_PROG	1	/* Use Programmable Clamp Pulse */
    u_short		:10;
  } b;
} pv_p_clamp_t;


typedef union {			
  u_short all;			/* please use word access only! */
  struct {
    u_short		:7;
    u_short plldata	:1;
    u_short		:8;
  } b;
} pv_pll_reg_t;

/* PLL-Register */
#define PV_PLL_SIZE	7	/* number of registers */

/* see Figure 3-8: PLL Serial Programming Timing */
#define PV_PLLADDR_MASK		0x0007
#define PV_PLLDATA_MASK		0x07FF
#define PV_PLLDATA_SHIFT	3
#define PV_PLL_READ	1
#define PV_PLL_WRITE	0

/* PLL-Register 0: FeedBack Divider */
/* Total Number of Pixels per Line - 1 */
#define PV_PLLREG0_ADDR	0
#define PV_PLLREG0_INIT	952 /* 907 */	/* CCIR: 64us * 14.18MHz clocks */
#define PV_FBKDIV_INIT	952
#define PV_FBKDIV_MASK	0x07FF
#define PV_FBKDIV_MIN	64
#define PV_FBKDIV_MAX	2048

/* PLL-Register 1: Additional Feedback Divider Low Byte (unused) */
#define PV_PLLREG1_ADDR	1
#define PV_PLLREG1_INIT	0x0

/* PLL-Register 2: Additional Feedback Divider High Byte (unused) */
#define PV_PLLREG2_ADDR	2
#define PV_PLLREG2_INIT	0x0

/* PLL-Register 3: Reference Divider */
/* PLL-Mode : always 0 */
/* XTAL-Mode: 14.318 MHz / Line Rate (kHz) */
#define PV_PLLREG3_ADDR	3
#define PV_PLLREG3_INIT	0x400	/* Only Negative Edge is supported */
#define PV_REFDIV_INIT	1
#define PV_REFDIV_MASK	0x03FF
#define PV_REFDIV_MIN	1
#define PV_REFDIV_MAX	1024

/* PLL-Register 4: Various Fuctions */
#define PV_PLLREG4_ADDR	4
#define PV_PLLREG4_INIT	0x075D	/* VCO Gain is 60MHz per Volt */
				/* PFD Gain is 15uA per 2Pi rad */
				/* PFD enabled */
				/* External Loop Filter selected */
				/* Internal VCO is used */
				/* OUT1 Clock feeds Feedback Divider */
/* VCO-Gain Selection */
#define PV_VCO_INIT	5
#define PV_VCO_MASK	0x0007
#define PV_VCO_10MHZ	0
#define PV_VCO_15MHZ	1
#define PV_VCO_20MHZ	2
#define PV_VCO_25MHZ	3
#define PV_VCO_45MHZ	4
#define PV_VCO_60MHZ	5
#define PV_VCO_75MHZ	6
#define PV_VCO_90MHZ	7

/* PLL-Register 5: Various functions */
#define PV_PLLREG5_ADDR	5
#define PV_PLLREG5_INIT	0x05E3	/* Internal Feedback Selected */
				/* Feedback Polarity is Negative Edge */ 
				/* Output Post Divide by 2 */
				/* Feedback Post Divide by 2 */
				/* Feedback will lead REF at Input to PFD */
				/* Fine Phase Adjust Disabled */
/* Output Post Scaler Selection */
#define PV_PDA_INIT	2
#define PV_PDA_MASK	0x0030
#define PV_PDA_SHIFT	4
#define PV_PDA_DIV_BY8	0
#define PV_PDA_DIV_BY4	1
#define PV_PDA_DIV_BY2	2
#define PV_PDA_DIV_BY1	3

/* PLL-Register 6: Various functions */
#define PV_PLLREG6_ADDR	6
#define PV_PLLREG6_INIT	0x0444	/* 6 lines - positive edge */
				/* OUT1 Select: Load Counter Output */
				/* OUT2 Select: Internal Feedback Pulse */
				/* OUT3 Select: Feedback Sync Pulse Low */
				/* OUT4 Select: Diff. Outp. Div. by 4 at 270 */
				/* Output Reset (disabled) */
				/* Output Test Mode (disabled) */
				/* Output Clock in Test Mode */
				/* External Reference Input Operation */
/* Load Counter*/
#define PV_LCNT_INIT	4
#define PV_LCNT_MASK	0x0007
#define PV_LCNT_3	0
#define PV_LCNT_4POS	1
#define PV_LCNT_4NEG	2
#define PV_LCNT_5	3
#define PV_LCNT_6POS	4
#define PV_LCNT_8NEG	5
#define PV_LCNT_8POS	6
#define PV_LCNT_10POS	7

struct pv_reg_t {
  pv_modid_t modid;
  u_short reserved_0;
  u_short reserved_1;
  u_short reserved_2;
  u_short lutaddr;
  u_short lutdata;
  u_short adcdata;
  u_short reserved_3;
  u_short htotal;
  u_short hsyncpw;
  u_short hequipw;
  u_short hserrpw;
  u_short vtotal;
  pv_vsyncpw_t vsyncpw;
  u_short vgstart;
  u_short vgwidth;
  u_short hoffset;
  u_short hactive;
  u_short voffset;
  u_short vactive;
  pv_strbdly_t strbdly;
  pv_frstdly_t frstdly;
  pv_con1_t con1;
  pv_con2_t con2;
  pv_trigcon_t trigcon;
  u_short sw_trig;
  pv_p_clamp_t p_clamp;
  pv_pll_reg_t pllprg;
  u_short reserved_4;
  u_short reserved_5;
  u_short reserved_6;
  u_short reserved_7;
};

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* linux && __KERNEL__ || !OS_DEFINED || __CONFIG__ */

#endif /* _AMPCVREG_H */

